class ToolsController < ApplicationController
  skip_before_action :authenticate_user!, only: [:welcome, :about]
  before_action :ensure_admin!, except: [:welcome, :about] 
  
  def welcome
    @mam = Guidepost.all.select{ |g| g.exists_in_kct? }.count
    @osm_guideposts = OsmGuidepost.all
  end
  
  def tool_index
  end
  
  def integrity_check
    @checks = [
      { table: :osm_signs, foreign_key: :osm_trail_id, from: :osm_trails },
      { table: :osm_signs, foreign_key: :osm_guidepost_id, from: :osm_guideposts },
      { table: :signs, foreign_key: :guidepost_id, from: :guideposts },
      { table: :signs, foreign_key: :trail_id, from: :trails },
      { table: :kct_kilometraz, foreign_key: :kct_trail_id, from: :kct_trails },
      { table: :kct_kilometraz, foreign_key: :kct_guidepost_id, from: :kct_guideposts },
      { table: :guideposts, foreign_key: :osmid, from: :osm_guideposts, primary_key: :osmid, options: :ignore_null },
    ].sort_by{|item| item[:table].to_s }.
      collect{|check| 
        check[:primary_key] ||= 'id'
        check[:delete_sql] = if check[:options] == :ignore_null
          "UPDATE `#{check[:table]}` SET `#{check[:table]}`.#{check[:foreign_key]} = NULL WHERE NOT EXISTS (SELECT * FROM #{check[:from]} WHERE #{check[:from]}.#{check[:primary_key]} = `#{check[:table]}`.#{check[:foreign_key]}) "
        else
          "DELETE FROM `#{check[:table]}` WHERE NOT EXISTS (SELECT * FROM #{check[:from]} WHERE #{check[:from]}.#{check[:primary_key]} = `#{check[:table]}`.#{check[:foreign_key]});"
        end
        check[:check_sql]  = if check[:options] == :ignore_null
          "SELECT * FROM `#{check[:table]}` WHERE (`#{check[:table]}`.#{check[:foreign_key]} IS NOT NULL) AND NOT EXISTS (SELECT * FROM #{check[:from]} WHERE #{check[:from]}.#{check[:primary_key]} = `#{check[:table]}`.#{check[:foreign_key]});"
        else 
          "SELECT * FROM `#{check[:table]}` WHERE NOT EXISTS (SELECT * FROM #{check[:from]} WHERE #{check[:from]}.#{check[:primary_key]} = `#{check[:table]}`.#{check[:foreign_key]});"
        end
        check[:output] =  ActiveRecord::Base.connection.execute(check[:check_sql])
        check[:dangling_count] = check[:output].size
        check
      }

    unless params[:check_index].nil? # action required
      check = @checks[params[:check_index].to_i]
      connection = ActiveRecord::Base.connection
      result = connection.execute(check[:delete_sql])
      flash.notice = "Cleaned #{result.inspect} dangling keys #{check[:foreign_key]} in #{check[:table]}"
      redirect_to :check_index=>nil
    end
  end
  
  def sot
    @osm_trails = OsmTrail.all
#     @osm_trails_array = @osm_trails.to_a
    
    @osm_guideposts = OsmGuidepost.all
#     @osm_guideposts_array = @osm_guideposts.to_a
    @osm_guideposts_trails_count = []
    @osm_guideposts_trails_count_example_guidepost = []
    
    total_pom = 0
    OsmGuidepost.joins(:osm_trails).select('osm_guideposts.*, COUNT(*) as osm_trails_count').group(:id).each do |osm_guidepost| 
      c = osm_guidepost.osm_trails_count
      total_pom = total_pom +1
      @osm_guideposts_trails_count[c] = 0 if @osm_guideposts_trails_count[c].nil?  
      @osm_guideposts_trails_count[c] += 1
      @osm_guideposts_trails_count_example_guidepost[c] = osm_guidepost
    end
    @osm_guideposts_trails_count[0] =+ (@osm_guideposts.count-total_pom)    
  end
  
  def statistic
    @osm_trails = OsmTrail.all
#     @osm_trails_array = @osm_trails.to_a

    @osm_trails_incomplete_count = 0
    @osm_trails_segments_count = []
    @osm_trails_segments_example_trails = 20.times.map{[]}
    @osm_trails.each do |osm_trail| 
      c = (osm_trail.gpx_segments || []).count
      @osm_trails_segments_count[c] = 0 if @osm_trails_segments_count[c].nil?  
      @osm_trails_segments_count[c] += 1
      if (@osm_trails_segments_example_trails[c] || []).size < 15 # max number of examples
        @osm_trails_segments_example_trails[c] << osm_trail
      end
      @osm_trails_incomplete_count += 1 if !osm_trail.osmtags.nil? and osm_trail.osmtags['complete']=='no'
    end

    @osm_trails_with_at_least_12_segments_count = (12..@osm_trails_segments_count.size-1).sum{|seg_count| @osm_trails_segments_count[seg_count]||0}
    @osm_trails_with_at_least_2_segments_count = (2..@osm_trails_segments_count.size-1).sum{|seg_count| @osm_trails_segments_count[seg_count]||0}

    @osm_guideposts = OsmGuidepost.all
#     @osm_guideposts_array = @osm_guideposts.to_a
    @osm_guideposts_trails_count = []
    @osm_guideposts_trails_count_example_guideposts = 20.times.map{[]}
    
    total_pom = 0
    OsmGuidepost.joins(:osm_trails).select('osm_guideposts.*, COUNT(*) as osm_trails_count').group(:id).each do |osm_guidepost| 
      c = osm_guidepost.osm_trails_count
      total_pom = total_pom +1
      @osm_guideposts_trails_count[c] = 0 if @osm_guideposts_trails_count[c].nil?  
      @osm_guideposts_trails_count[c] += 1
      @osm_guideposts_trails_count_example_guideposts[c] << osm_guidepost unless @osm_guideposts_trails_count_example_guideposts[c].size > 10 and c<5 
      @osm_guideposts_trails_count_example_guideposts[c]
    end
    @osm_guideposts_trails_count[0] =+ (@osm_guideposts.count-total_pom)

    total_pom = 0
    @kct_guideposts_trails_count=[]
    @kct_guideposts_trails_count_example_guideposts = 20.times.map{[]}

    KctGuidepost.joins(:kct_trails).select('kct_guideposts.*, COUNT(DISTINCT kct_trails.kctref) as kct_trails_count').group(:id).each do |kct_guidepost|
      c = kct_guidepost.kct_trails_count
      total_pom = total_pom +1
      @kct_guideposts_trails_count[c] = 0 if @kct_guideposts_trails_count[c].nil?  
      @kct_guideposts_trails_count[c] += 1
      @kct_guideposts_trails_count_example_guideposts[c] << kct_guidepost unless @kct_guideposts_trails_count_example_guideposts[c].size > 10 and c<5
    end
    @kct_guideposts_trails_count[0] =+ (KctGuidepost.all.count-total_pom)
  end
  
  def recount_priorities
#     @trails_co_zasahuji_do_plzenskeho_kraje = OsmTrail.all.select{|ot| !ot.osm_guideposts.select{|og| og.guidepost.andand.kctref.andand.match(/^P(S|J)/)}.empty? }
    
    @trails_co_zasahuji_do_plzenskeho_kraje = OsmTrail.all.select{|ot| ot.priority.to_f > 0}
    
    @trails_co_zasahuji_do_plzenskeho_kraje.each{|t| 
      if !t.error.blank? 
        t.priority = 120
      elsif !t.exists?
        t.priority = 115
      elsif t.gpx_segments.andand.size == 1
        t.priority = 110
      else
        t.priority = 10
      end
      t.save
    }
    redirect_to :action => 'tool_index'
  end
  
  def graphs
  end

end
